VersorInfo

Struct VersorInfo 

pub struct VersorInfo {
    pub parity: VersorParity,
    pub is_unit: bool,
    pub sandwich_targets: Vec<String>,
}
Information about a verified versor type.

Fields

parity: VersorParity

The parity of the versor (even or odd).

is_unit: bool

Whether this is a unit versor (V * rev(V) = 1).

Unit versors have a simpler sandwich product formula since V⁻¹ = rev(V) for unit versors.

sandwich_targets: Vec<String>

Types that this versor can transform via sandwich product.

A type can be transformed if the sandwich product V * X * rev(V) produces output of the same grades as the input.