pub struct ProductGenerator<'a> { /* private fields */ }Expand description
Generates the products.rs file.
Note: This generator now produces an essentially empty file. All product implementations are in traits.rs.
Implementations
Sourceimpl<'a> ProductGenerator<'a>
impl<'a> ProductGenerator<'a>
Sourcepub fn new(
spec: &'a AlgebraSpec,
_algebra: &'a Algebra,
_table: ProductTable,
) -> Self
pub fn new( spec: &'a AlgebraSpec, _algebra: &'a Algebra, _table: ProductTable, ) -> Self
Creates a new product generator.
Sourcepub fn generate_products_file(&self) -> TokenStream
pub fn generate_products_file(&self) -> TokenStream
Generates the complete products.rs file.
Note: As of PRD-24, all product implementations are inlined into trait implementations in traits.rs. This file is now empty except for the header.