GradeConflict

Struct GradeConflict 

pub struct GradeConflict {
    pub grade_a: usize,
    pub grade_b: usize,
    pub reason: String,
}
Represents a conflict between two grades in the geometric constraint.

When grades grade_a and grade_b have a conflict, it means that elements with non-zero components in both grades will produce non-canceling cross-terms in u * ũ.

Fields

grade_a: usize

First grade in the conflict.

grade_b: usize

Second grade in the conflict.

reason: String

Description of why these grades conflict.