ParseError

Enum ParseError 

pub enum ParseError {
Show 23 variants Toml(Error), DimensionTooLarge(usize), EmptySignature, DuplicateBasisName(String), InvalidBasisName { name: String, }, BasisIndexOutOfBounds { name: String, index: usize, dim: usize, }, NonContiguousBasisIndices { expected: usize, found: usize, name: String, }, InvalidGrade { type_name: String, grade: usize, max: usize, }, FieldCountMismatch { type_name: String, expected: usize, got: usize, }, DuplicateFieldName { type_name: String, field: String, }, DuplicateTypeName(String), InvalidBladeName { name: String, }, BladeIndexOutOfBounds { name: String, index: usize, dim: usize, }, UnknownType(String), SelfAlias { type_name: String, }, AliasCycle { type_name: String, }, UnknownFieldName { type_name: String, field: String, }, InvalidValue { field: String, value: String, expected: String, }, SparseBladeCountMismatch { type_name: String, blades: usize, fields: usize, }, SparseBladeGradeMismatch { type_name: String, blade: String, blade_grade: usize, grades: Vec<usize>, }, IncompleteAlgebra { name: String, count: usize, details: String, }, MissingFieldMap { type_name: String, }, FieldMapGradeMismatch { type_name: String, field: String, blade: String, blade_grade: usize, grades: Vec<usize>, },
}
Errors that can occur when parsing an algebra specification.

Variants



Toml(Error)

TOML syntax error.



DimensionTooLarge(usize)

Algebra dimension exceeds the maximum supported (6).



EmptySignature

Signature must have at least one basis vector.



DuplicateBasisName(String)

Duplicate basis vector name in signature.



InvalidBasisName

Invalid basis vector name in signature.

Fields

name: String

The invalid basis name.



BasisIndexOutOfBounds

Basis vector index exceeds algebra dimension.

Fields

name: String

The basis name.

index: usize

The invalid index (1-based from name).

dim: usize

Algebra dimension.



NonContiguousBasisIndices

Basis vector indices are not contiguous (0, 1, 2, …).

Fields

expected: usize

The expected index.

found: usize

The found index.

name: String

The basis name with the wrong index.



InvalidGrade

Type references a grade that exceeds the algebra dimension.

Fields

type_name: String

The type name.

grade: usize

The invalid grade.

max: usize

Maximum valid grade.



FieldCountMismatch

Type has wrong number of fields for its grades.

Fields

type_name: String

The type name.

expected: usize

Expected number of fields.

got: usize

Actual number of fields.



DuplicateFieldName

Duplicate field name in a type.

Fields

type_name: String

The type name.

field: String

The duplicate field name.



DuplicateTypeName(String)

Duplicate type name in specification.



InvalidBladeName

Invalid blade name in blades section.

Fields

name: String

The invalid blade name.



BladeIndexOutOfBounds

Blade index exceeds algebra dimension.

Fields

name: String

The blade name.

index: usize

The invalid index.

dim: usize

Algebra dimension.



UnknownType(String)

Unknown type reference (e.g., in alias_of).



SelfAlias

Type alias references itself.

Fields

type_name: String

The type name.



AliasCycle

Type alias forms a cycle.

Fields

type_name: String

The type name.



UnknownFieldName

Unknown field name in a type’s field list.

Fields

type_name: String

The type name.

field: String

The unknown field name.



InvalidValue

Invalid value for a configuration field.

Fields

field: String

The field name.

value: String

The invalid value.

expected: String

Description of expected values.



SparseBladeCountMismatch

Sparse type blade count doesn’t match field count.

Fields

type_name: String

The type name.

blades: usize

Number of blade mappings.

fields: usize

Number of fields.



SparseBladeGradeMismatch

Sparse type blade doesn’t match specified grades.

Fields

type_name: String

The type name.

blade: String

The blade name.

blade_grade: usize

The blade’s actual grade.

grades: Vec<usize>

The expected grades.



IncompleteAlgebra

Algebra is incomplete: some products don’t have matching output types.

Fields

name: String

Algebra name.

count: usize

Number of missing products.

details: String

Human-readable details of missing products.



MissingFieldMap

Type is missing required field_map.

Fields

type_name: String

The type name.



FieldMapGradeMismatch

Blade grade in field_map doesn’t match type’s grades.

Fields

type_name: String

The type name.

field: String

The field name.

blade: String

The blade name.

blade_grade: usize

The blade’s actual grade.

grades: Vec<usize>

The expected grades.