pub struct GroebnerCache { /* private fields */ }Expand description
Cache for Groebner bases computed for type pairs.
Computing Groebner bases can be expensive, so we cache them for reuse when the same type pair is encountered multiple times.
Implementations
Sourceimpl GroebnerCache
impl GroebnerCache
Sourcepub fn get_or_compute(
&mut self,
type_a: &TypeSpec,
type_b: &TypeSpec,
collector: &ProductConstraintCollector<'_>,
) -> Arc<GroebnerSimplifier>
pub fn get_or_compute( &mut self, type_a: &TypeSpec, type_b: &TypeSpec, collector: &ProductConstraintCollector<'_>, ) -> Arc<GroebnerSimplifier>
Gets or computes a Groebner simplifier for a type pair.
If the simplifier for this pair is already cached, returns it. Otherwise, computes the Groebner basis and caches it.